import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.MessageFormat; // um die Meldungen zu formatieren
import java.net.URL;            // um die URL vom Konstruktor ans Spielfeld weiterzugeben


// s. auch Spielfeld.java, Snake.java, Feld.java
public class NibblesGame extends Frame { // das Spiel
	
	protected Spielfeld sf;           // das Spielfeld
	protected GameThread gameThread;  // der Thread des Spiels
	protected int level;              // aktueller Level
	protected int ziffer;             // gerade zu erreichende Zahl
	protected boolean pause;          // ob gerade Pause ist
	protected String status;          // Status-Meldung an die Spieler
	protected ResourceBundle rBundle; // Proto-Texte fuer die Meldungen
	
				
	protected final long delay = 300; // Geschwindigkeit des Spiels
	
	protected class GameThread extends Thread { // laesst das Spiel laufen
		public GameThread() { super(); }
		public void run() {
			for(;;) { // einfach immer weitermachen...
				if (!pause) {
					Move(player1); // erste Schlange bewegen
					if (player2 != null) Move(player2); // und 2.
					if ((player1.getLeben() == 0) && ((player2 == null) || (player2.getLeben() == 0)))
					   Quit(); // aufhoeren, falls keine Leben mehr uebrig
				}
				try { sleep(delay); } catch (InterruptedException e) {}
			}
		}
	}		
	
	protected class Player { // verwaltet einen Spieler
		protected Snake snake; // seine Schlange
		protected String name; // sein Name
		protected int number;  // Nummer der Schlange = Konstante aus Spielfeld
		protected int punkte;  // seine Punkte
		protected int leben;   // seine Leben
		
		public Player(String name, int number) {
			this.name = name;
			this.number = number;
			leben = 5;     // 5 Leben am Anfang
			punkte = 0;    // und natuerlich keine Punkte
		}
		
		public String getName() { return name; }
		public int getPunkte() { return punkte; }
		public int getLeben() { return leben; }
		
		public void ChangeDirection(int dir) {
			// nicht aendern, falls entgegengesetzt zur jetzigen Richtung
			// Richtungen sind 0..3
			if ( (dir + 2) % 4 != snake.getDirection() )
			  snake.setDirection(dir);
		}
		
		public int MoveSnake() { // gibt zurueck, was passiert
			int oldTailx = snake.getTailx(); // alten Schwanz sichern
			int oldTaily = snake.getTaily();
			snake.Move(); // Schlange bewegen
			int trifft = sf.getFeldTyp(snake.getHeadx(), snake.getHeady());
			if (trifft < 11) { // ueberlebt: Nichts oder Ziffer getroffen
				if (trifft < 10) {
					punkte += level*trifft; // Ziffer getroffen
					snake.Elongate(trifft+2); // verlaengern
					ShowPlayerStatus();     // Status updaten
				}
				// Kopf neu zeichnen
				ChangeFeld(snake.getHeadx(), snake.getHeady(), number);
			}
			else {  // TOD
				leben--;
				KillSnake();        // alte Schlange vernichten
				ShowPlayerStatus(); // Status updaten
			}
			if ((snake.getTailx() != oldTailx) || (snake.getTaily() != oldTaily))
			// falls sich der Schwanz bewegte, den alten Schwanz uebermalen
				ChangeFeld(oldTailx, oldTaily, Spielfeld.nichts);
			return trifft; // zurueckgeben, was getroffen wurde
		}
		
		protected void KillSnake() {
			// loescht die Schlange, wenn sie tot ist
			for(SnakeLink sl = snake.getTail(); sl.getPrev() != null; ) {
				int x = sl.getx(); int y  = sl.gety();
				ChangeFeld(x, y, Spielfeld.nichts);
				sl = sl.getPrev();
			}
		}	
		
		public void StartNewSnake() {
			// startet eine neue Schlange
			int startx, starty;
			int startdir = Snake.hoch; // am Anfang nach oben
			// erst mal Startposition suchen
			do {
				// Random - Startposition
				startx = (int) Math.floor((Math.random() * Spielfeld.width));
				starty = (int) Math.floor((Math.random() * Spielfeld.height));
			}
			// Feld muss leer sein und auch Feld darueber sollte leer sein
			while ((sf.getFeldTyp(startx, starty) != Spielfeld.nichts) 
			    || (sf.getFeldTyp(startx, starty - 1) != Spielfeld.nichts));
			snake = new Snake(startx, starty);  // Schlange erzeugen
			ChangeFeld(startx, starty, number); // Kopf zeichnen
		}
	}
	
	protected Player player1, player2; // die Spieler
	
	protected void Move(Player p) { // wird nur aufgerufen, wenn p != null
		if (p.getLeben() > 0) { // sonst macht's keinen Sinn
			int returnCode = p.MoveSnake();
			if (returnCode < 10) { // Ziffer getroffen
				if (returnCode == 9) { // neuen Level starten
					Object args[] = { new Integer(level), new Integer(++level) };
					MsgBox(MessageFormat.format(rBundle.getString("DoneLevel"), args), "Nibbles");
					StartNewLevel();
				} else ShowNewDigit(); // neue Ziffer zeigen
			} else if (returnCode > 10) { //stirbt
				// neue Schlange vor MsgBox wg. Reaktionszeit
				if (p.getLeben() > 0) p.StartNewSnake();
				Object[] args = { p.getName(), new Integer(p.getLeben()) };
				MsgBox(MessageFormat.format(rBundle.getString("Tod"), args), "Nibbles");
			}
		}
	}
	
	protected void ChangeFeld(int x, int y, int typ) { // aendert und uebermalt ein Feld
		sf.setFeldAt(x, y, typ); // Feld aendern
		// nur dieses Feld neu zeichnen
		repaint(x * Feld.width + Feld.xOffset, y * Feld.height + Feld.yOffset, Feld.width, Feld.height);
	}
	
	public void MsgBox(String msg, String title) { // zeigt Meldung an
		MessageBox msgBox = new MessageBox(msg, title); // s. MessageBox.java
	}
	
	public void Pause() {
		// Pause im Spiel
		FontMetrics fm = this.getToolkit().getFontMetrics(this.getFont());
		int lineHeight = fm.getLeading() + fm.getMaxAscent() + fm.getMaxDescent() + 20; // 20 zur Sicherheit
		pause = true;
		int lineLength = fm.stringWidth(status) + 200; // 200, um sicherzugehen
		// der Pause - String ist auf jeden Fall kuerzer
		status = rBundle.getString("Pause");
		// Status - Zeile neu zeichnen
		repaint(0, Spielfeld.height * Feld.height + Feld.yOffset, lineLength, lineHeight);
	}
	
	public void ShowPlayerStatus() {
		// Zeigt Information zum Spieler an
		Object[] args1 = { new Integer(level), player1.getName(), new Integer(player1.getLeben()), new Integer(player1.getPunkte()) };
		status = MessageFormat.format(rBundle.getString("Status1"), args1);
		if (player2 != null) {
			Object[] args2 = { player2.getName(), new Integer(player2.getLeben()), new Integer(player2.getPunkte()) };
			status += MessageFormat.format(rBundle.getString("Status2"), args2);
		}
		FontMetrics fm = this.getToolkit().getFontMetrics(this.getFont());
		int lineHeight = fm.getLeading() + fm.getMaxAscent() + fm.getMaxDescent() + 20;
		// Status - Zeile neu zeichnen
		repaint(0, Spielfeld.height * Feld.height + Feld.yOffset, fm.stringWidth(status) + 200, lineHeight);
	}
	
	protected void ShowNewDigit() {
		// zeigt eine neue zu erreichende Ziffer
		int digitx, digity;
		do {
			// Random - Position fuer die Ziffer
			digitx = (int) Math.floor(Math.random() * Spielfeld.width);
			digity = (int) Math.floor(Math.random() * Spielfeld.height);
		}
		// aber Feld muss vorher natuerlich leer sein
		while (sf.getFeldTyp(digitx, digity) != Spielfeld.nichts);
		ChangeFeld(digitx, digity, ++ziffer);
	}
	
	public void StartNewLevel() {
		// Faengt einen neuen Level an
		Pause(); // Spiel anhalten
		// im Augenblick hat's 9 Level, danach wird immer der 9. benutzt
		try { if (level < 10) sf.LoadLevel(level); else sf.LoadLevel(9);}
		catch (Spielfeld.LevelLoadException e) {
			if (e.getMsg() == "Keine Datei!") MsgBox(rBundle.getString("DateiFehlt"), rBundle.getString("FehlerTitel"));
			else if (e.getMsg() == "Falsches Format!") MsgBox(rBundle.getString("FormatFalsch"), rBundle.getString("FehlerTitel"));
			else if (e.getMsg() == "I/O - Error!") MsgBox(rBundle.getString("IOError"), rBundle.getString("FehlerTitel"));
			else if (e.getMsg() == "Falsche URL!") MsgBox(rBundle.getString("URLFalsch"), rBundle.getString("FehlerTitel"));
			else MsgBox(rBundle.getString("LevelFehler"), rBundle.getString("FehlerTitel"));
			Quit(); // bei Fehler beim Laden Spiel beenden
		}
		repaint(); // alles neu zeichnen
		// neue Schlangen starten
		if (player1.getLeben() > 0) player1.StartNewSnake();
		if ((player2 != null) && (player2.getLeben() > 0)) player2.StartNewSnake();
		ziffer = 0;         // wird in ShowNewDigit() auf 1 erhoeht
		ShowNewDigit();     // erste Ziffer anzeigen
		ShowPlayerStatus(); // Status anzeigen
		pause = false;      // und los geht's
	}
	
	public void Quit() {
		pause = true; // anhalten
		// Nachricht mit letztem Spielstatus anzeigen
		Object[] args1 = { new Integer(level), player1.getName(), new Integer(player1.getPunkte()) };
		String msg = MessageFormat.format(rBundle.getString("Ende1"), args1);
		if (player2 != null) {
			Object[] args2 = { player2.getName(), new Integer(player2.getPunkte()) };
			msg += MessageFormat.format(rBundle.getString("Ende2"), args2);
		}
		MsgBox(msg, "Nibbles");
		// und beenden
		dispose(); 
	}
	
	public NibblesGame(String name1, String name2, int startLevel, Locale l, URL baseURL) {
		this(name1, startLevel, l, baseURL); // anderen Konstruktor aufrufen
		player2 = new Player(name2, Spielfeld.schlange2); // 2. Spieler starten
	}
	
	public NibblesGame(String name1, int startLevel, Locale l, URL baseURL) {
		super();
		sf = new Spielfeld(baseURL); // Spielfeld initialisieren
		// Resourcen stehen in "NibblesGameResources*.properties"
		rBundle = ResourceBundle.getBundle("NibblesGameResources", l);
		level = startLevel;
		gameThread = new GameThread();
		pause = true; // erst mal nicht gleich loslegen
		status = "";
		gameThread.start();
		player1 = new Player(name1, Spielfeld.schlange1); // 1. Spieler starten
		player2 = null; // bleibt null falls nur 1 Spieler
		this.addFocusListener(new FocusListener() {
			// falls Fokus verlorengeht, Pause einlegen bis er wiederkommt
			public void focusLost(FocusEvent e) { Pause(); }
			public void focusGained(FocusEvent e) {
				ShowPlayerStatus();
				pause = false;
			}
		});
		this.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
		                switch (e.getKeyCode()) {
		                  // mit p wird die Pause umgeschaltet
		                  case KeyEvent.VK_P:     if (pause) {
		                  	                    pause = false; 
		                  	                    ShowPlayerStatus();
		                  	                  } else Pause(); 
		                                          break;
			          // Cursortasten steuern 1. Spieler
			          case KeyEvent.VK_UP:    player1.ChangeDirection(Snake.hoch); 
			                                  break;
			          case KeyEvent.VK_LEFT:  player1.ChangeDirection(Snake.links); 
			                                  break;
			          case KeyEvent.VK_DOWN:  player1.ChangeDirection(Snake.runter); 
			                                  break;
			          case KeyEvent.VK_RIGHT: player1.ChangeDirection(Snake.rechts); 
			                                  break;
			          // w,a,s,d steuern 2. Spieler
			          case KeyEvent.VK_W:     if (player2 != null)
			                                  player2.ChangeDirection(Snake.hoch);
			                                  break;
			          case KeyEvent.VK_A:     if (player2 != null)
			                                  player2.ChangeDirection(Snake.links);
			                                  break;
			          case KeyEvent.VK_S:     if (player2 != null)
			                                  player2.ChangeDirection(Snake.runter);
			                                  break;
			          case KeyEvent.VK_D:     if (player2 != null)
			                                  player2.ChangeDirection(Snake.rechts);
			                                  break;
			          // mit q wird das Spiel beendet
			          case KeyEvent.VK_Q:     Quit();
			        }
		        }
		});
		this.addWindowListener(new WindowAdapter() {
			// Spiel beenden, wenn Fenster geschlossen wird
			public void windowClosing(WindowEvent e) { Quit(); }
		});
		this.setSize(720, 500);          // Fenster gross genug machen
		this.setResizable(false);        // sollte auch so gross bleiben
		this.setBackground(Color.white); // Nichts ist auch weiss
	}
	
	public void paint(Graphics g) { 
		sf.PaintAll(g);            // Spielfeld zeichnen
		g.setColor(Color.white);   // alte Statusmeldung uebermalen
		g.fillRect(Feld.xOffset, Spielfeld.height * Feld.height + Feld.yOffset, Spielfeld.width*Feld.width, 30);
		g.setColor(Color.black);   // neuen Status ausgeben
		g.drawString(status, Feld.xOffset, (Spielfeld.height+1)*Feld.height + Feld.yOffset);
	}
	
	public void update(Graphics g) { paint(g); } // nicht alles loeschen beim update
}
