import java.awt.*;
import java.io.*;
import java.net.*; // um die Level zu laden

public class Spielfeld { // verwaltet die ganzen Felder

        protected static final int width  = 79;  // Breite
        protected static final int height = 22;  // Hoehe
        
        // Konstanten fuer die Art eines Feldes
        public static final int nichts    = 10;
        public static final int mauer     = 11;
        public static final int schlange1 = 12;
        public static final int schlange2 = 13;
        // 1 - 9 sind die Zahlen
	
	protected Feld[][] feld; // die Felder
	
	protected URL baseURL;   // wo alle Level liegen
	
	public Spielfeld(URL baseURL) {
		this.baseURL = baseURL;
	        feld = new Feld[width][height];
	}
	
	public Feld getFeldAt(int x, int y) { return feld[x][y]; }
	
	public void setFeldAt(int x, int y, int f) { 
		switch(f) { // f muss natuerlich aus 1..schlange2 sein
			case schlange1: feld[x][y] = new Schlange1(x,y); break;
			case schlange2: feld[x][y] = new Schlange2(x,y); break;
			case mauer:     feld[x][y] = new Mauer(x,y); break;
			case nichts:    feld[x][y] = new Nichts(x,y); break;
			default:        feld[x][y] = new Zahl(x, y, f);
		}
	}
	
	public int getFeldTyp(int x, int y) { // was in einem Feld drin ist
		if (feld[x][y] instanceof Nichts) return nichts;
		else if (feld[x][y] instanceof Mauer) return mauer;
		else if (feld[x][y] instanceof Zahl) return ((Zahl)feld[x][y]).getNumber();
		else if (feld[x][y] instanceof Schlange1) return schlange1;
		else return schlange2; // bleibt ja sonst nichts mehr uebrig
	}
	
	
	protected void PaintNr(Graphics g, int x, int y) { 
		if (feld[x][y] != null) feld[x][y].Paint(g); // einfach weiterreichen
	}
	
	public void PaintAll(Graphics g) { // alle Felder zeichnen
		for (int i = 0; i < width; i++)
		  for (int j = 0; j < height; j++) PaintNr(g, i, j);
	}
	
	public void LoadLevel(int levelnr) throws LevelLoadException { // die Exception s.u.
		BufferedReader in; // liest den Level
		try { 
			// Leveldateien haben die Form ?level.nbs
			URL levelURL = new URL(baseURL, levelnr + "level.nbs");
			in = new BufferedReader(new InputStreamReader(levelURL.openStream()));
		}
		catch (MalformedURLException e) { throw new LevelLoadException("Falsche URL!"); }
		catch (FileNotFoundException e) { throw new LevelLoadException("Keine Datei!"); }
		catch (IOException e) { throw new LevelLoadException("I/O - Error!"); }
		try {
		     for (int y = 0; y < height; y++) {
			     // zeilenweise einlesen
			     String line = in.readLine();
			     if ((line == null) || (line.length() != width)) 
			         throw new LevelLoadException("Falsches Format!");
			     for (int x = 0; x < width; x++)
			        // Code: 0 <=> Nichts; anderes <=> Mauer
			        if (line.charAt(x) == '0') feld[x][y] = new Nichts(x,y);
			        else feld[x][y] = new Mauer(x,y);
		     }
	        }
	        catch (IOException e) { throw new LevelLoadException("I/O - Error!"); }
	        finally { 
	        	try { in.close(); } 
	        	catch (IOException e) { throw new LevelLoadException("I/O - Error!"); } 
	        }
	}
	
	public class LevelLoadException extends Exception {
		// fasst alle Exceptions, die beim Laden auftreten koennen, zusammen
		protected String msg; // Message der Exception
		public LevelLoadException() { super(); msg = ""; }
		public LevelLoadException(String s) { super(s); msg = s; }
		public String getMsg() { return msg; }
	}
}
